/*
   (c) Copyright 2000-2002  convergence integrated media GmbH.
   All rights reserved.

   Written by Denis Oliver Kropp <dok@convergence.de>,
              Andreas Hundt <andi@convergence.de> and
              Sven Neumann <sven@convergence.de>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the
   Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.
*/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <iostream>

#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>

#include <dfb++.h>

#include "dfbapp.h"
#include "dfbimage.h"

class ShowImageApp : public DFBApp {
public:
     ShowImageApp (std::string filename) {
          m_filename = filename;
     }

     virtual ~ShowImageApp() {}

protected:
     /* called after initialization */
     virtual void Setup (int width, int height) {
          m_image.LoadImage (m_filename);
     }

     /* render callback */
     virtual void Render (IDirectFBSurface *surface) {
          int x = ((int) surface->GetWidth()  - (int) m_image.GetWidth())  / 2;
          int y = ((int) surface->GetHeight() - (int) m_image.GetHeight()) / 2;

          surface->Blit( m_image, NULL, x, y );
     }

     /* update application state, return true to block until events arrive */
     virtual bool Update () {
          return true;
     }

     /* event callback, return true to exit application */
     virtual bool HandleEvent (DFBInputEvent &event) {
          switch (event.type) {
               case DIET_KEYPRESS:
                    if (event.key_symbol == DIKS_ESCAPE)
                         return true;
                    break;

               default:
                    break;
          }

          return false;
     }

private:
     std::string m_filename;
     DFBImage    m_image;
};

int main(int argc, char *argv[])
{
     ShowImageApp app (argv[1]);

     try {
          app.Init(&argc, &argv);

          app.Run();
     }
     catch (DFBException *ex) {
          std::cerr << std::endl;
          std::cerr << "Caught exception!" << std::endl;
          std::cerr << "  -- " << ex << std::endl;
     }

     return 0;
}

